#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Event handler for the share of logext package."""


from ionas.request import (Entry,)

import libevh.evhcmd as evhcmd
from fal_info import (Results, ReqTypes, FAL_LOGEXT_PATH)


EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"

USBD_CATEGORY = "usbd"
LOGEXT_NAME = "logext"


class EventHandler(evhcmd.EvhCmd):
    """Event Handler for the share of logext package."""

    def __handle_usbd_event(self, eventinfo, entry):
        """Handle usbd event."""

        self.send_entry(Entry(ReqTypes.UPDATE, LOGEXT_NAME),
                        FAL_LOGEXT_PATH).get()
        entry.set_result(Results.OK)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == USBD_CATEGORY:
                return self.__handle_usbd_event(eventinfo, entry)
            else:
                return self.__handle_usbd_event(eventinfo, entry)
        except:
            # Ignore all error.
            self.exception("logext usb key plug/unplug")
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    EventHandler(option=evhcmd.EvhCmdOption()).handle_request()
