#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Event handler for the manual of logext package."""

import libevh.evhcmd as evhcmd
import evh_config as config
from fal_info import (Results, ReqTypes)
from libevh.util import (set_manual, regist_man_set_event,
                            get_handler_real_path)
from pkgaddon.logext_config import (MAN_NAME,)

EVENTINFO_NAME = "eventinfo"
VOLUME_NAME_PATH = "volume/name"
DEVICE_NAME_PATH = "device/name"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

VOLUME_CATEGORY = "volume"
MOUNT_ACTION = "mount"
MKFS_ACTION = "mkfs"

DEV_KEY = "dev"


class EventHandler(evhcmd.EvhCmd):
    """Event Handler for the manual of logext package."""

    def __handle_volume_mount_event(self, eventinfo, entry):
        """Handle volume:mount event."""

        if eventinfo.is_set_to_true(PREVIOUS_ATTR):
            return

        if (eventinfo.get_node(VOLUME_NAME_PATH).get_text() !=
                config.INTERNAL_VOL):
            return

        set_manual(entry, [MAN_NAME], get_handler_real_path(__file__))

    def __handle_volume_mkfs_event(self, eventinfo, entry):
        """Handle volume:mkfs event."""

        if eventinfo.is_set_to_true(PREVIOUS_ATTR):
            return

        internal_dev = config.VOL_INFO[config.INTERNAL_VOL][DEV_KEY]
        enc_dev = config.DEV_TO_ENC_DEV[internal_dev]
        if (eventinfo.get_node(DEVICE_NAME_PATH).get_text() != internal_dev and
            eventinfo.get_node(DEVICE_NAME_PATH).get_text() != enc_dev):

            return

        regist_man_set_event(entry, get_handler_real_path(__file__))

    def __handle_volue_event(self, eventinfo, entry):
        """Handle volume event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)

        if action == MOUNT_ACTION:
            self.__handle_volume_mount_event(eventinfo, entry)

        elif action == MKFS_ACTION:
            self.__handle_volume_mkfs_event(eventinfo, entry)

        entry.set_result(Results.OK)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == VOLUME_CATEGORY:
                return self.__handle_volue_event(eventinfo, entry)
        except:
            # Non target events
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    EventHandler(option=evhcmd.EvhCmdOption()).handle_request()
