# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command

from ionas.request import Entry, XmlNode, XmlConfNode
from fal_info import Results, FAL_LOGEXT_PATH
from datetime import time


class FALLogExt(Command):

    COMMAND = FAL_LOGEXT_PATH

    def load(self):

        def __dict(entry):
            d = {}
            try:
                logext_node = entry.get()

                try:
                    d["restriction"] = {}
                    d["restriction"]["logshare"] = logext_node.get_node("restrictionlist/restriction[@type='logshare']").is_enabled()
                    d["restriction"]["logsetting"] = logext_node.get_node("restrictionlist/restriction[@type='logsetting']").is_enabled()
                except:
                    d["restriction"] = {}
                    d["restriction"]["logshare"] = None
                    d["restriction"]["logsetting"] = None

                try:
                    d["eventlogconf"] = {}
                    d["eventlogconf"]["enabled"] = logext_node.get_node("eventlogconf").is_enabled()
                    #d["eventlogconf"]["autodelete"] = {}
                    #d["eventlogconf"]["autodelete"]["enabled"] = logext_node.get_node("eventlogconf/autodelete").is_enabled()
                    #d["eventlogconf"]["autodelete"]["expire"] = {}
                    #d["eventlogconf"]["autodelete"]["expire"]["days"] = logext_node.get_value("eventlogconf/autodelete/expire/days")
                    #d["eventlogconf"]["autodelete"]["expire"]["percent"] = logext_node.get_value("eventlogconf/autodelete/expire/percent")
                except:
                    d["eventlogconf"] = {}
                    d["eventlogconf"]["enabled"] = None
                    d["eventlogconf"]["autodelete"] = {}
                    d["eventlogconf"]["autodelete"]["enabled"] = None
                    d["eventlogconf"]["autodelete"]["expire"] = {}
                    d["eventlogconf"]["autodelete"]["expire"]["days"] = None
                    d["eventlogconf"]["autodelete"]["expire"]["percent"] = None

                try:
                    d["accesslogconf"] = {}
                    d["accesslogconf"]["enabled"] = logext_node.get_node("accesslogconf").is_enabled()
                    d["accesslogconf"]["autodelete"] = {}
                    d["accesslogconf"]["autodelete"]["enabled"] = logext_node.get_node("accesslogconf/autodelete").is_enabled()
                    d["accesslogconf"]["autodelete"]["expire"] = {}
                    d["accesslogconf"]["autodelete"]["expire"]["days"] = logext_node.get_value("accesslogconf/autodelete/expire/days")
                    d["accesslogconf"]["autodelete"]["expire"]["percent"] = logext_node.get_value("accesslogconf/autodelete/expire/percent")
                    d["accesslogconf"]["sharelist"] = []
                    for share_node in logext_node.get_node_list("accesslogconf/sharelist/share"):
                        d["accesslogconf"]["sharelist"].append({
                            "name": share_node.get_value("name"),
                        })
                    d["accesslogconf"]["shareall"] = logext_node.get_node("accesslogconf/sharelist").is_set_to_true("all")
                except:
                    d["accesslogconf"] = {}
                    d["accesslogconf"]["enabled"] = None
                    d["accesslogconf"]["autodelete"] = {}
                    d["accesslogconf"]["autodelete"]["enabled"] = None
                    d["accesslogconf"]["autodelete"]["expire"] = {}
                    d["accesslogconf"]["autodelete"]["expire"]["days"] = None
                    d["accesslogconf"]["autodelete"]["expire"]["percent"] = None
                    d["accesslogconf"]["sharelist"] = []
                    d["accesslogconf"]["shareall"] = None

                try:
                    d["share"] = {}
                    d["share"]["name"] = logext_node.get_value("share/name")
                    d["share"]["volume"] = logext_node.get_value("share/volume")
                    d["share"]["path"] = logext_node.get_value("share/path")
                    d["share"]["desc"] = logext_node.get_value("share/desc")
                    d["share"]["permission"] = {}
                    d["share"]["permission"]["readonly"] = logext_node.get_node("share/permission/readonly").is_enabled()
                    d["share"]["permission"]["detailpermission"] = {}
                    d["share"]["permission"]["detailpermission"]["enabled"] = logext_node.get_node("share/permission/detailpermission").is_enabled()
                    d["share"]["permission"]["detailpermission"]["readonlymember"] = {}
                    d["share"]["permission"]["detailpermission"]["readonlymember"]["userlist"] = []
                    for user_node in logext_node.get_node_list("share/permission/detailpermission/readonlymember/userlist/user"):
                        d["share"]["permission"]["detailpermission"]["readonlymember"]["userlist"].append({
                            "name": user_node.get_value("name"),
                        })
                    d["share"]["permission"]["detailpermission"]["readonlymember"]["grouplist"] = []
                    for group_node in logext_node.get_node_list("share/permission/detailpermission/readonlymember/grouplist/group"):
                        d["share"]["permission"]["detailpermission"]["readonlymember"]["grouplist"].append({
                            "name": group_node.get_value("name"),
                        })
                    d["share"]["permission"]["detailpermission"]["readwritemember"] = {}
                    d["share"]["permission"]["detailpermission"]["readwritemember"]["userlist"] = []
                    for user_node in logext_node.get_node_list("share/permission/detailpermission/readwritemember/userlist/user"):
                        d["share"]["permission"]["detailpermission"]["readwritemember"]["userlist"].append({
                            "name": user_node.get_value("name"),
                        })
                    d["share"]["permission"]["detailpermission"]["readwritemember"]["grouplist"] = []
                    for group_node in logext_node.get_node_list("share/permission/detailpermission/readwritemember/grouplist/group"):
                        d["share"]["permission"]["detailpermission"]["readwritemember"]["grouplist"].append({
                            "name": group_node.get_value("name"),
                        })
                except:
                    d["share"] = {}
                    d["share"]["name"] = None
                    d["share"]["volume"] = None
                    d["share"]["path"] = None
                    d["share"]["desc"] = None
                    d["share"]["permission"] = {}
                    d["share"]["permission"]["readonly"] = None
                    d["share"]["permission"]["detailpermission"] = {}
                    d["share"]["permission"]["detailpermission"]["enabled"] = None
                    d["share"]["permission"]["detailpermission"]["readonlymember"] = {}
                    d["share"]["permission"]["detailpermission"]["readonlymember"]["userlist"] = []
                    d["share"]["permission"]["detailpermission"]["readonlymember"]["grouplist"] = []
                    d["share"]["permission"]["detailpermission"]["readwritemember"] = {}
                    d["share"]["permission"]["detailpermission"]["readwritemember"]["userlist"] = []
                    d["share"]["permission"]["detailpermission"]["readwritemember"]["grouplist"] = []

                try:
                    d["alertlist"] = []
                    for alert_node in logext_node.get_node_list("alertlist/alert"):
                        adict = {
                            "name": alert_node.get_value("name"),
                            #"username": alert_node.get_value("username"),
                            "usernamelist": [n.get_text() for n in alert_node.get_node_list("usernamelist/username")],
                            "usernamelist_all": alert_node.get_node("usernamelist").is_set_to_true("all"),
                            "usernamelist_not": alert_node.get_node("usernamelist").is_set_to_true("not"),
                            "hostname": alert_node.get_value("hostname"),
                            "hostname_not": alert_node.get_node("hostname").is_set_to_true("not"),
                            "ipaddress": alert_node.get_value("ipaddr"),
                            "ipaddress_not": alert_node.get_node("ipaddr").is_set_to_true("not"),
                            "operation": dict([(operation_node.get_attr_value("type"), operation_node.is_enabled()) for operation_node in alert_node.get_node_list("operationlist/operation")]),
                            "result": alert_node.get_value("result"),
                            "sharename": alert_node.get_value("sharename"),
                            "sharename_not": alert_node.get_node("sharename").is_set_to_true("not"),
                            "period": {
                                "weekdays": {
                                    "sun": alert_node.get_node("period/weekdays").is_set_to_true("sun"),
                                    "mon": alert_node.get_node("period/weekdays").is_set_to_true("mon"),
                                    "tue": alert_node.get_node("period/weekdays").is_set_to_true("tue"),
                                    "wed": alert_node.get_node("period/weekdays").is_set_to_true("wed"),
                                    "thu": alert_node.get_node("period/weekdays").is_set_to_true("thu"),
                                    "fri": alert_node.get_node("period/weekdays").is_set_to_true("fri"),
                                    "sat": alert_node.get_node("period/weekdays").is_set_to_true("sat"),
                                },
                                #"time": {
                                #    "from": time(int(alert_node.get_value("period/from/hour")),
                                #                 int(alert_node.get_value("period/from/minute"))),
                                #    "to": time(int(alert_node.get_value("period/to/hour")),
                                #               int(alert_node.get_value("period/to/minute"))),
                                #},
                            },
                            "period_not": alert_node.get_node("period").is_set_to_true("not"),
                            "path": alert_node.get_value("path"),
                        }
                        adict["period"]["time"] = {}
                        try:
                            adict["period"]["time"]["from"] = time(int(alert_node.get_value("period/from/hour")),
                                                                   int(alert_node.get_value("period/from/minute")))
                        except:
                            adict["period"]["time"]["from"] = None
                        try:
                            adict["period"]["time"]["to"] = time(int(alert_node.get_value("period/to/hour")),
                                                                 int(alert_node.get_value("period/to/minute")))
                        except:
                            adict["period"]["time"]["to"] = None
                        d["alertlist"].append(adict)
                except:
                    d["alertlist"] = []

            except:
                pass
            return d

        try:
            entry = self._load()
            return __dict(entry)
        except:
            return {}

    def save(self, data=None, log=True, messages={}):
        __default_messages = {
            Results.OK: ["MSG_LOGEXT_CHANGE", {}],
            None: ["ERR_LOGEXT_CHANGE", {}],
        }
        __default_messages.update(messages)

        try:
            entry = self._save(data=data)
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def apply(self, data=None, log=True, messages={}):
        from pkgaddon.logext_config import INVALID_USB_MEMORY_ERROR, ALREADY_REGISTERED_ERROR
        __default_messages = {
            Results.OK: ["MSG_LOGEXT_CHANGE", {}],
            Results.AUTH_ERROR: ["ERR_LOGEXT_CHANGE_NO_KEY", {}],
            Results.DEVICE_NOT_FOUND: ["ERR_LOGEXT_NO_USB_MEMORY", {}],
            INVALID_USB_MEMORY_ERROR: ["ERR_LOGEXT_INVALID_USB_MEMORY", {}],
            ALREADY_REGISTERED_ERROR: ["ERR_LOGEXT_REGISTERED_USB_MEMORY", {}],
            None: ["ERR_LOGEXT_CHANGE", {}],
        }
        __default_messages.update(messages)

        try:
            entry = self._apply(data=data)
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def register(self, log=True, messages={}):
        from pkgaddon.logext_config import INVALID_USB_MEMORY_ERROR, ALREADY_REGISTERED_ERROR

        __default_messages = {
            Results.OK: ["MSG_LOGEXT_REGISTER_KEY", {}],
            Results.AUTH_ERROR: ["ERR_LOGEXT_REGISTER_NO_KEY", {}],
            Results.DEVICE_NOT_FOUND: ["ERR_LOGEXT_NO_USB_MEMORY", {}],
            INVALID_USB_MEMORY_ERROR: ["ERR_LOGEXT_INVALID_USB_MEMORY", {}],
            ALREADY_REGISTERED_ERROR: ["ERR_LOGEXT_REGISTERED_USB_MEMORY", {}],
            None: ["ERR_LOGEXT_REGISTER_KEY", {}],
        }
        __default_messages.update(messages)

        try:
            entry = self._register()
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def list(self):
        try:

            def __list(entry):
                l = []
                try:
                    for key_node in entry.get().get_node_list("keylist/key"):
                        l.append({
                            "connected": key_node.get_value("usb/@port", None),
                            "vid": key_node.get_value("usb/vid"),
                            "pid": key_node.get_value("usb/pid"),
                            "manufacturer": key_node.get_value("usb/manufacturer"),
                            "product": key_node.get_value("usb/product"),
                            "rev": key_node.get_value("usb/rev"),
                            "serial": key_node.get_value("usb/serial"),
                        })
                except:
                    pass
                return l

            entry = self._list()
            return __list(entry)
        except:
            return {}

    def unregister(self, vid, pid, serial, log=True, messages={}):
        __default_messages = {
            Results.OK: ["MSG_LOGEXT_UNREGISTER_KEY", {}],
            Results.AUTH_ERROR: ["ERR_LOGEXT_UNREGISTER_NO_KEY", {}],
            Results.DEVICE_NOT_FOUND: ["ERR_LOGEXT_NO_USB_MEMORY", {}],
            None: ["ERR_LOGEXT_UNREGISTER_KEY", {}],
        }
        __default_messages.update(messages)

        try:
            entry = self._unregister(vid, pid, serial)
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def _load(self):
        entry = Entry("load", """<logext/>""")
        return self.request(entry)

    def _save(self, data=None):
        entry = Entry("save", """<logext/>""")
        return self.request(entry)

    def _apply(self, data=None):
        entry = Entry("apply", """<logext/>""")
        logext_node = entry.get()

        if "restrictionlist" in data:
            try:
                logext_node.append_child("restrictionlist")
                restrictionlist_node = logext_node.get_node("restrictionlist")
                for _type in ("logshare", "logsetting"):
                    restriction_node = XmlConfNode("<restriction/>")
                    restriction_node.set_attrs({"type": _type})
                    restriction_node.enable(data["restrictionlist"][_type]["enabled"])
                    restrictionlist_node.append_child(restriction_node)
            except:
                pass

        if "eventlogconf" in data:
            try:
                logext_node.append_child("eventlogconf")
                eventlogconf_node = logext_node.get_node("eventlogconf")
                eventlogconf_node.enable(data["eventlogconf"]["enabled"])
                #eventlogconf_node.append_child("autodelete")
                #autodelete_node = eventlogconf_node.get_node("autodelete")
                #autodelete_node.enable(data["eventlogconf"]["autodelete"]["enabled"])
                #autodelete_node.append_child("expire")
                #expire_node = autodelete_node.get_node("expire")
                #expire_node.append_child("days", text=str(data["eventlogconf"]["autodelete"]["expire"]["days"]))
                #expire_node.append_child("percent", text=str(data["eventlogconf"]["autodelete"]["expire"]["percent"]))
            except:
                pass

        if "accesslogconf" in data:
            try:
                logext_node.append_child("accesslogconf")
                accesslogconf_node = logext_node.get_node("accesslogconf")
                accesslogconf_node.enable(data["accesslogconf"]["enabled"])
                accesslogconf_node.append_child("autodelete")
                autodelete_node = accesslogconf_node.get_node("autodelete")
                autodelete_node.enable(data["accesslogconf"]["autodelete"]["enabled"])
                autodelete_node.append_child("expire")
                expire_node = autodelete_node.get_node("expire")
                expire_node.append_child("days", text=str(data["accesslogconf"]["autodelete"]["expire"]["days"]))
                expire_node.append_child("percent", text=str(data["accesslogconf"]["autodelete"]["expire"]["percent"]))
                accesslogconf_node.append_child("sharelist")
                sharelist_node = accesslogconf_node.get_node("sharelist")
                sharelist_node.set_bool_attr("all", data["accesslogconf"]["shareall"])
                for share in data["accesslogconf"]["sharelist"]:
                    share_node = XmlNode("<share/>")
                    share_node.append_child("name", text=share["name"])
                    sharelist_node.append_child(share_node)
            except:
                pass

        if "share" in data:
            try:
                logext_node.append_child("share")
                share_node = logext_node.get_node("share")
                share_node.append_child("name", text=data["share"]["name"])
                share_node.append_child("permission")
                permission_node = share_node.get_node("permission")
                #permission_node.append_child("readonly")
                #readonly_node = permission_node.get_node("readonly")
                #readonly_node.enable(data["share"]["permission"]["readonly"])
                permission_node.append_child("detailpermission")
                detailpermission_node = permission_node.get_node("detailpermission")

                detailpermission_node.append_child("readonlymember")
                readonlymember_node = detailpermission_node.get_node("readonlymember")
                readonlymember_node.append_child("userlist")
                userlist_node = readonlymember_node.get_node("userlist")
                for user in data["share"]["permission"]["detailpermission"]["readonlymember"]["userlist"]:
                    user_node = XmlNode("<user/>")
                    user_node.append_child("name", text=user["name"])
                    userlist_node.append_child(user_node)
                readonlymember_node.append_child("grouplist")
                grouplist_node = readonlymember_node.get_node("grouplist")
                for group in data["share"]["permission"]["detailpermission"]["readonlymember"]["grouplist"]:
                    group_node = XmlNode("<group/>")
                    group_node.append_child("name", text=group["name"])
                    grouplist_node.append_child(group_node)

                detailpermission_node.append_child("readwritemember")
                readwritemember_node = detailpermission_node.get_node("readwritemember")
                readwritemember_node.append_child("userlist")
                userlist_node = readwritemember_node.get_node("userlist")
                for user in data["share"]["permission"]["detailpermission"]["readwritemember"]["userlist"]:
                    user_node = XmlNode("<user/>")
                    user_node.append_child("name", text=user["name"])
                    userlist_node.append_child(user_node)
                readwritemember_node.append_child("grouplist")
                grouplist_node = readwritemember_node.get_node("grouplist")
                for group in data["share"]["permission"]["detailpermission"]["readwritemember"]["grouplist"]:
                    group_node = XmlNode("<group/>")
                    group_node.append_child("name", text=group["name"])
                    grouplist_node.append_child(group_node)
            except:
                pass

        if "alertlist" in data:
            """
            alertlist: {
                "alert": {
                    "name": アクセスアラート名,
                    "username": アクセス元のユーザー名,
                    "username_not": True or False,
                    "hostname": アクセス元のホスト名,
                    "hostname_not": True or False,
                    "ipaddress": アクセス元のIPアドレス,
                    "ipaddress_not": True or False,
                    "operationlist": {
                        type: True or False,
                        type: True or False,
                        ...
                    },
                    "result": "ok" or "fail",
                    "sharename": アクセスがあった共有フォルダー名,
                    "path": アクセスがあったパス,
                    "period": {
                        "not": True or False,
                        "weekdays": {
                            "mon": True or False,
                            ...
                            "sun": True or False,
                        },
                        "time": {
                            "from": Time(から),
                            "to": Time(まで),
                        },
                    },
                },
            },
            """
            try:
                logext_node.append_child("alertlist")
                alertlist_node = logext_node.get_node("alertlist")
                for alert in data["alertlist"]:
                    alert_node = XmlConfNode("<alert/>")
                    alert_node.append_child("name", text=alert["name"])

                    #alert_node.append_child("username", text=alert["username"])
                    #username_node = alert_node.get_node("username")
                    #username_node.set_bool_attr("not", alert["username_not"])
                    alert_node.append_child("usernamelist")
                    usernamelist_node = alert_node.get_node("usernamelist")
                    for username in alert["usernamelist"]:
                        usernamelist_node.append_child("username", text=username)
                    usernamelist_node.set_bool_attr("all", alert.get("usernamelist_all", False))
                    usernamelist_node.set_bool_attr("not", alert.get("usernamelist_not", False))

                    alert_node.append_child("hostname", text=alert["hostname"])
                    hostname_node = alert_node.get_node("hostname")
                    hostname_node.set_bool_attr("not", alert["hostname_not"])
                    alert_node.append_child("ipaddr", text=alert["ipaddress"])
                    ipaddr_node = alert_node.get_node("ipaddr")
                    ipaddr_node.set_bool_attr("not", alert["ipaddress_not"])
                    alert_node.append_child("operationlist")
                    operationlist_node = alert_node.get_node("operationlist")
                    for operation, enabled in alert["operation"].iteritems():
                        operation_node = XmlConfNode("<operation/>")
                        operation_node.set_attrs({"type": operation})
                        operation_node.enable(enabled)
                        operationlist_node.append_child(operation_node)
                    alert_node.append_child("result", text=alert["result"])
                    alert_node.append_child("sharename", text=alert["sharename"])
                    sharename_node = alert_node.get_node("sharename")
                    sharename_node.set_bool_attr("not", alert["sharename_not"])
                    alert_node.append_child("period")
                    period_node = alert_node.get_node("period")
                    period_node.set_bool_attr("not", alert["period_not"])
                    period_node.append_child("weekdays")
                    weekdays_node = period_node.get_node("weekdays")
                    weekdays_node.set_attrs(dict([(k, str(v).lower()) for k, v in alert["period"]["weekdays"].iteritems()]))
                    period_node.append_child("from")
                    from_node = period_node.get_node("from")
                    from_time = alert["period"]["time"]["from"]
                    try:
                        from_node.append_child("hour", text=str(from_time.hour))
                    except:
                        from_node.append_child("hour")
                    try:
                        from_node.append_child("minute", text=str(from_time.minute))
                    except:
                        from_node.append_child("minute")
                    period_node.append_child("to")
                    to_node = period_node.get_node("to")
                    to_time = alert["period"]["time"]["to"]
                    try:
                        to_node.append_child("hour", text=str(to_time.hour))
                    except:
                        to_node.append_child("hour")
                    try:
                        to_node.append_child("minute", text=str(to_time.minute))
                    except:
                        to_node.append_child("minute")
                    alert_node.append_child("path", text=alert["path"])
                    alertlist_node.append_child(alert_node)
            except:
                pass

        return self.request(entry)

    def _register(self):
        entry = Entry("register", """<logext/>""")
        return self.request(entry)

    def _list(self):
        entry = Entry("list", """<logext/>""")
        return self.request(entry)

    def _unregister(self, vid, pid, serial):
        entry = Entry("unregister", """<logext><key><usb/></key></logext>""")
        logext_node = entry.get()
        usb_node = logext_node.get_node("key/usb")
        usb_node.append_child("vid", text=vid)
        usb_node.append_child("pid", text=pid)
        usb_node.append_child("serial", text=serial)
        return self.request(entry)


if __name__ == u"__main__":
    fal = FALLogExt()

    """
    print fal.load()
    """

    """
    print fal.register()
    print fal.list()
    for key in fal.list():
        print fal.unregister(key["vid"], key["pid"], key["serial"])
    print fal.list()
    """
    
    """
    print fal.apply(data={
        "alertlist": [
            {
                "name": "abc",
                "username": "user1",
                "username_not": True,
                "hostname": "host1",
                "hostname_not": True,
                "ipaddress": "192.168.123.234",
                "ipaddress_not": True,
                "operation": {"connect": True},
                "result": "ok",
                "sharename": "disk1",
                "sharename_not": False,
                "path": "file1",
            },
        ],
    })
    print fal.load()
    """
