# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.forms.formsets import formset_factory
import os
from datetime import time
from project.view import FuncView
import settings

from fields import (AccesslogAlertNameField,
                    AccesslogAlertUsernameField,
                    AccesslogAlertUsernameAllField,
                    AccesslogAlertUsernameNotField,
                    AccesslogAlertHostnameField,
                    AccesslogAlertHostnameNotField,
                    AccesslogAlertIPAddressField,
                    AccesslogAlertIPAddressNotField,
                    AccesslogAlertOperationField,
                    AccesslogAlertResultField,
                    AccesslogAlertSharenameField,
                    AccesslogAlertSharenameNotField,
                    AccesslogAlertTimestampDaysField,
                    AccesslogAlertTimestampTime1Field,
                    AccesslogAlertTimestampTime2Field,
                    AccesslogAlertTimestampNotField,
                    AccesslogAlertPathField)

from project.fal.logext import FALLogExt

try:
    from pkgaddon.logext_config import ALERT_CONDITION_MAX as MAX_NUM
except:
    MAX_NUM = 30


class AlertForm(forms.Form):

    name = AccesslogAlertNameField()
    hostname = AccesslogAlertHostnameField()
    hostname_not = AccesslogAlertHostnameNotField()
    ipaddress = AccesslogAlertIPAddressField()
    ipaddress_not = AccesslogAlertIPAddressNotField()
    operation = AccesslogAlertOperationField()
    result = AccesslogAlertResultField()
    ts_days = AccesslogAlertTimestampDaysField()
    ts_time1 = AccesslogAlertTimestampTime1Field()
    ts_time2 = AccesslogAlertTimestampTime2Field()
    ts_not = AccesslogAlertTimestampNotField()
    path = AccesslogAlertPathField()

    def __init__(self, *args, **kwargs):
        super(AlertForm, self).__init__(*args, **kwargs)
        self.fields['usernamelist'] = AccesslogAlertUsernameField()
        self.fields['usernamelist_all'] = AccesslogAlertUsernameAllField()
        self.fields['usernamelist_not'] = AccesslogAlertUsernameNotField()
        self.fields['sharename'] = AccesslogAlertSharenameField()
        self.fields['sharename_not'] = AccesslogAlertSharenameNotField()

    def __raise_ValidationError_if_empty_userlist(self, userlist_all, userlist):
        if userlist or userlist_all:
            pass
        else:
            raise forms.ValidationError(_(u"This field is required."))

    def clean_usernamelist_all(self):
        userlist_all = self.cleaned_data.get("usernamelist_all", None)
        userlist = self.cleaned_data.get("usernamelist", None)
        if userlist_all is not None and userlist is not None:
            self.__raise_ValidationError_if_empty_userlist(userlist_all, userlist)
        return userlist_all

    def clean_usernamelist(self):
        userlist_all = self.cleaned_data.get("usernamelist_all", None)
        userlist = self.cleaned_data.get("usernamelist", None)
        if userlist_all is not None and userlist is not None:
            self.__raise_ValidationError_if_empty_userlist(userlist_all, userlist)
        return userlist

    def clean(self):
        data = self.cleaned_data
        #if isinstance(data["ts_time1"], time) and isinstance(data["ts_time2"], time):
        #    t1 = data["ts_time1"]
        #    t2 = data["ts_time2"]
        #    if ((t2.hour * 60 + t2.minute) - (t1.hour * 60 + t1.minute)) < 1:
        #        raise forms.ValidationError(_(u"LogExt: Time is invalid."))
        return data


AlertFormset = formset_factory(AlertForm,
                               extra=0,
                               can_delete=True,
                               max_num=MAX_NUM)


class AlertView(FuncView):

    views = ['apply']

    def view(self, request):
        formset = AlertFormset(prefix='alert', initial=self.__initial())

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'formset': formset,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        formset = AlertFormset(prefix='alert', data=request.POST)

        with self.processing(constname="ERR_LOGEXT_STATE_CHANGE") as p:
            if formset.is_valid():
                name_is_unique = True
                names = []
                alertlist = []
                for form in formset:
                    if form.is_valid():
                        if form.cleaned_data == {}:
                            # formsetを使うと追加分のformはempty_permitted=Trueになるため
                            continue
                        if form.cleaned_data["DELETE"]:
                            # 削除フラグがたっているformでもformset.cleaned_dataに入ってくるため
                            continue
                        name = form.cleaned_data["name"]
                        if name in names:  # 重複チェック
                            name_is_unique = False
                            break
                        names.append(name)
                        weekdays = [d.lower() for d in form.cleaned_data["ts_days"]]
                        alertlist.append({
                            "name": form.cleaned_data["name"],
                            "usernamelist": form.cleaned_data["usernamelist"],
                            "usernamelist_all": form.cleaned_data["usernamelist_all"],
                            "usernamelist_not": form.cleaned_data["usernamelist_not"],
                            "hostname": form.cleaned_data["hostname"],
                            "hostname_not": form.cleaned_data["hostname_not"],
                            "ipaddress": form.cleaned_data["ipaddress"],
                            "ipaddress_not": form.cleaned_data["ipaddress_not"],
                            "operation": dict([(o, True) for o in form.cleaned_data["operation"]]),
                            "result": form.cleaned_data["result"],
                            "sharename": form.cleaned_data["sharename"],
                            "sharename_not": form.cleaned_data["sharename_not"],
                            "path": form.cleaned_data["path"],
                            "period": {
                                "weekdays": dict([(d, d in weekdays) for d in ["sun", "mon", "tue", "wed", "thu", "fri", "sat"]]),
                                "time": {
                                    "from": form.cleaned_data["ts_time1"],
                                    "to": form.cleaned_data["ts_time2"],
                                },
                            },
                            "period_not": form.cleaned_data["ts_not"],
                        })
                if name_is_unique:
                    fal = FALLogExt()
                    p.result(fal.apply(data={"alertlist": alertlist}))
                    self.set_message(fal.get_message())
                    formset = AlertFormset(prefix='alert', initial=self.__initial())
                else:
                    self.set_message(_(u"LogExt: Please input unique name."))

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'formset': formset,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __initial(self):
        try:
            initials = []
            settings = FALLogExt().load()["alertlist"]
            for setting in settings:
                initials.append({
                    "name": setting["name"],
                    "usernamelist": setting["usernamelist"],
                    "usernamelist_all": setting["usernamelist_all"],
                    "usernamelist_not": setting["usernamelist_not"],
                    "hostname": setting["hostname"],
                    "hostname_not": setting["hostname_not"],
                    "ipaddress": setting["ipaddress"],
                    "ipaddress_not": setting["ipaddress_not"],
                    "operation": [k for k, v in setting["operation"].iteritems() if v],
                    "result": setting["result"],
                    "sharename": setting["sharename"],
                    "sharename_not": setting["sharename_not"],
                    "ts_days": [k for k, v in setting["period"]["weekdays"].iteritems() if v],
                    "ts_time1": setting["period"]["time"]["from"],
                    "ts_time2": setting["period"]["time"]["to"],
                    "ts_not": setting["period_not"],
                    "path": setting["path"],
                })
            return initials
        except:
            return []
