# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
import os
import re
from project.view import FuncView
import settings

from project.forms.fields.checkbox import CheckboxField
from django.forms import IntegerField
from fields import RequireKeyForConfigurationRadioField
from fields import RequireKeyForSharingRadioField
from fields import ForbiddenFormatRadioField
from fields import SaveAccesslogCheckboxField
from fields import OutputAccesslogToShareCheckboxField
from fields import OutputSystemlogToShareCheckboxField
from fields import AutoDeleteAccesslogFileCheckboxField
from fields import AutoDeleteSystemlogFileCheckboxField
from fields import RemainingCapacityForAccesslogTextField
from fields import RemainingCapacityForSystemlogTextField
from fields import PeriodForAccesslogTextField
from fields import PeriodForSystemlogTextField
from fields import ShareMultipleCheckboxField
from fields import AccesslogTargetIsAllShareCheckboxField
#from fields import LogFolderNameTextField
from fields import LogFolderNameField
from project.forms.fields.extras.share import ShareNameField
from fields import DenyUsersField, ROUsersField, RWUsersField
from fields import DenyGroupsField, ROGroupsField, RWGroupsField

from project.fal.logext import FALLogExt
from systemlog import FALSystemlog


class ConfForm(forms.Form):
   
    require_key_c = RequireKeyForConfigurationRadioField()
    require_key_s = RequireKeyForSharingRadioField()
    #forbidden_format = ForbiddenFormatRadioField()

    accesslog_enabled = SaveAccesslogCheckboxField()
    accesslog_output_to_share = OutputAccesslogToShareCheckboxField()
    accesslog_auto_delete = AutoDeleteAccesslogFileCheckboxField()
    accesslog_remaining_capacity = RemainingCapacityForAccesslogTextField()
    accesslog_period = PeriodForAccesslogTextField()

    systemlog_output_to_share = OutputSystemlogToShareCheckboxField()
    #systemlog_auto_delete = AutoDeleteSystemlogFileCheckboxField()
    #systemlog_remaining_capacity = RemainingCapacityForSystemlogTextField()
    #systemlog_period = PeriodForSystemlogTextField()

    def __init__(self, *args, **kwargs):
        super(ConfForm, self).__init__(*args, **kwargs)

        self.fields["accesslog_sharelist"] = ShareMultipleCheckboxField()
        self.fields["accesslog_target_is_all"] = AccesslogTargetIsAllShareCheckboxField()

        #self.fields["logfoldername"] = LogFolderNameTextField()
        #self.fields["logfoldername"] = ShareNameField(label=_(u"LogExt: Log Folder Name"))
        self.fields["logfoldername"] = LogFolderNameField(label=_(u"LogExt: Log Folder Name"))
        self.fields["ro_users"] = ROUsersField()
        self.fields["rw_users"] = RWUsersField()
        selected_u = []
        selected_u += self["ro_users"].value() or []
        selected_u += self["rw_users"].value() or []
        self.fields["deny_users"] = DenyUsersField(exclude=selected_u)
        self.fields["ro_groups"] = ROGroupsField()
        self.fields["rw_groups"] = RWGroupsField()
        selected_g = []
        selected_g += self["ro_groups"].value() or []
        selected_g += self["rw_groups"].value() or []
        self.fields["deny_groups"] = DenyGroupsField(exclude=selected_g)


class ConfView(FuncView):

    views = ['apply']

    #SHARENAME_SUFFIX = "$"

    def view(self, request):
        form = ConfForm(initial=self.__initial())

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        form = ConfForm(data=request.POST)

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                result = True

                systemlog = FALSystemlog()
                accesslog_enabled_old = systemlog.load()["accesslog"]
                if systemlog.apply(data={"accesslog": form.cleaned_data["accesslog_enabled"]}, log=True):
                    logext = FALLogExt()
                    if logext.apply({
                        "restrictionlist": {
                            "logshare": {
                                "enabled": form.cleaned_data["require_key_s"],
                            },
                            "logsetting": {
                                "enabled": form.cleaned_data["require_key_c"],
                            },
                        },
                        "share": {
                            "name": "%s$" % form.cleaned_data["logfoldername"],
                            "permission": {
                                "detailpermission": {
                                    "readonlymember": {
                                        "userlist": [{"name": name} for name in form.cleaned_data["ro_users"]],
                                        "grouplist": [{"name": name} for name in form.cleaned_data["ro_groups"]],
                                    },
                                    "readwritemember": {
                                        "userlist": [{"name": name} for name in form.cleaned_data["rw_users"]],
                                        "grouplist": [{"name": name} for name in form.cleaned_data["rw_groups"]],
                                    },
                                },
                            },
                        },
                        "accesslogconf": {
                            "enabled": form.cleaned_data["accesslog_output_to_share"],
                            "autodelete": {
                                "enabled": form.cleaned_data["accesslog_auto_delete"],
                                "expire": {
                                    "days": form.cleaned_data["accesslog_period"],
                                    "percent": form.cleaned_data["accesslog_remaining_capacity"],
                                },
                            },
                            "shareall": form.cleaned_data["accesslog_target_is_all"],
                            "sharelist": [{"name": name} for name in form.cleaned_data["accesslog_sharelist"]],
                        },
                        "eventlogconf": {
                            "enabled": form.cleaned_data["systemlog_output_to_share"],
                            #"autodelete": {
                            #    "enabled": form.cleaned_data["systemlog_auto_delete"],
                            #    "expire": {
                            #        "days": form.cleaned_data["systemlog_period"],
                            #        "percent": form.cleaned_data["systemlog_remaining_capacity"],
                            #    },
                            #},
                        },
                    }):
                        self.set_message(logext.get_message())
                        result = True
                    else:
                        self.set_message(logext.get_message())
                        result = False
                        systemlog.apply(data={"accesslog": accesslog_enabled_old}, log=False)
                else:
                    self.set_message(systemlog.get_message())
                    result = False

                p.result(result)

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __initial(self):
        try:
            data = FALLogExt().load()
            return {
                "require_key_c": data["restriction"]["logsetting"],
                "require_key_s": data["restriction"]["logshare"],

                "accesslog_enabled": FALSystemlog().load()["accesslog"],
                "accesslog_output_to_share": data["accesslogconf"]["enabled"],
                "accesslog_auto_delete": data["accesslogconf"]["autodelete"]["enabled"],
                "accesslog_remaining_capacity": data["accesslogconf"]["autodelete"]["expire"]["percent"],
                "accesslog_period": data["accesslogconf"]["autodelete"]["expire"]["days"],
                "accesslog_sharelist": [share["name"] for share in data["accesslogconf"]["sharelist"]],
                "accesslog_target_is_all": data["accesslogconf"]["shareall"],

                "systemlog_output_to_share": data["eventlogconf"]["enabled"],
                #"systemlog_auto_delete": data["eventlogconf"]["autodelete"]["enabled"],
                #"systemlog_remaining_capacity": data["eventlogconf"]["autodelete"]["expire"]["percent"],
                #"systemlog_period": data["eventlogconf"]["autodelete"]["expire"]["days"],

                "logfoldername": re.sub("\$$", "", data["share"]["name"]),
                "ro_users": [user["name"] for user in data["share"]["permission"]["detailpermission"]["readonlymember"]["userlist"]],
                "rw_users": [user["name"] for user in data["share"]["permission"]["detailpermission"]["readwritemember"]["userlist"]],
                "ro_groups": [user["name"] for user in data["share"]["permission"]["detailpermission"]["readonlymember"]["grouplist"]],
                "rw_groups": [user["name"] for user in data["share"]["permission"]["detailpermission"]["readwritemember"]["grouplist"]],
            }
        except:
            return {}
