# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
import os
from project.view import FuncView
import settings

from fields import KeylistField

from project.fal.logext import FALLogExt


class KeyForm(forms.Form):

    def __init__(self, choices=[], *args, **kwargs):
        super(KeyForm, self).__init__(*args, **kwargs)
        self.fields["keylist"] = KeylistField(choices=choices)


class KeyView(FuncView):

    SEPARATOR = ','

    views = ['register', 'unregister']

    def view(self, request):
        keylist, form = self.__get_keylist_and_form()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
            'keylist': keylist,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_register(self, request):
        keylist, form = self.__get_keylist_and_form(data=request.POST)

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                fal = FALLogExt()
                p.result(fal.register())
                keylist, form = self.__get_keylist_and_form()
                self.set_message(fal.get_message())

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
            'keylist': keylist,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_unregister(self, request):
        keylist, form = self.__get_keylist_and_form(data=request.POST)

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                result = True
                for key in form.cleaned_data["keylist"]:
                    vid, pid, serial = key.split(self.SEPARATOR)[:3]
                    fal = FALLogExt()
                    result = result and fal.unregister(vid, pid, serial)
                    self.set_message(fal.get_message())
                p.result(result)
                keylist, form = self.__get_keylist_and_form()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
            'keylist': keylist,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __get_keylist_and_form(self, data=None):
        try:
            keylist = FALLogExt().list()

            form = KeyForm(data=data,
                           choices=[(self.SEPARATOR.join([k["vid"], k["pid"], k["serial"]]), "%s" % i) for i, k in enumerate(keylist)])
            for key, cb in zip(keylist, form["keylist"]):
                key["cb"] = cb
            return keylist, form
        except:
            return [], KeyForm(data=data)
